;; Generated package description from engrave-faces.el  -*- no-byte-compile: t -*-
(define-package "engrave-faces" "0.3.1.0.20240421.82802" "Convert font-lock faces to other formats" '((emacs "27.1")) :commit "0cc5d0b9801944d5678a7908885f9dea8de05753" :authors '(("TEC" . "https://git.tecosaur.net/")) :maintainer '("TEC" . "contact@tecosaur.net") :keywords '("faces") :url "https://github.com/tecosaur/engrave-faces")
